//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/Specular/SpecularMagneticStrategy.h
//! @brief     Defines namespace Compute::SpecularMagnetic.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2020
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif

#ifndef USER_API
#ifndef BORNAGAIN_RESAMPLE_SPECULAR_SPECULARMAGNETICSTRATEGY_H
#define BORNAGAIN_RESAMPLE_SPECULAR_SPECULARMAGNETICSTRATEGY_H

#include "Sample/Multilayer/RoughnessModels.h"
#include "Base/Vector/Vectors3D.h"
#include <Eigen/Core>
#include <memory>
#include <vector>

class IFlux;
class SliceStack;
using Fluxes = std::vector<std::unique_ptr<const IFlux>>;

//! Implements the magnetic Fresnel computation with Nevot-Croce roughness
//!
//! Implements the transfer matrix formalism for the calculation of wave
//! amplitudes of the coherent wave solution in a multilayer with magnetization.
//! For a description, see internal
//! document "Polarized Implementation of the Transfer Matrix Method"
namespace Compute::SpecularMagnetic {

    //! Computes refraction angle reflection/transmission coefficients
    //! for given sliced multilayer and wavevector k
    Fluxes Execute(const SliceStack& slices, const kvector_t& k);

    //! Computes the Fresnel R coefficient for the top layer only
    //! Introduced in order to speed up pure reflectivity computations
    Eigen::Matrix2cd
    computeTopLayerR(const SliceStack& slices, const std::vector<complex_t>& kzs,
                     const RoughnessModel&);
} // namespace Compute::SpecularMagnetic

#endif // BORNAGAIN_RESAMPLE_SPECULAR_SPECULARMAGNETICSTRATEGY_H
#endif // USER_API
