//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/Specular/SpecularScalarStrategy.h
//! @brief     Defines namespace Compute::SpecularScalar.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif

#ifndef USER_API
#ifndef BORNAGAIN_RESAMPLE_SPECULAR_SPECULARSCALARSTRATEGY_H
#define BORNAGAIN_RESAMPLE_SPECULAR_SPECULARSCALARSTRATEGY_H

#include "Base/Vector/Vectors3D.h"
#include "Sample/Multilayer/RoughnessModels.h"
#include <memory>
#include <vector>

class IFlux;
class SliceStack;
using Fluxes = std::vector<std::unique_ptr<const IFlux>>;

//! Implements the scalar Fresnel computation
//!
//! Implements method 'execute' to compute refraction angles and transmission/reflection
//! coefficients for coherent wave propagation in a multilayer.
//!
//! Inherited by SpecularScalarNCStrategy, SpecularScalarTanhStrategy

namespace Compute::SpecularScalar {

    //! Computes refraction angles and transmission/reflection coefficients
    //! for given coherent wave propagation in a multilayer.
    Fluxes Execute(const SliceStack& slices, const kvector_t& k);

    //! Computes the Fresnel R coefficient for the top layer only.
    //! Introduced in order to speed up pure reflectivity computations.
    complex_t
    computeTopLayerR(const SliceStack& slices, const std::vector<complex_t>& kz,
                     const RoughnessModel& r_model);

} // namespace Compute::SpecularScalar

#endif // BORNAGAIN_RESAMPLE_SPECULAR_SPECULARSCALARSTRATEGY_H
#endif // USER_API
